"""
Control complex python object through strings.
Wrap functions so that the complex arguments can be specified through a string key
that points to the actual python object (which is stored in a session's memory or
persisted in some fashion).
"""

from warnings import warn

warn(f'Module has moved to front.crude')

from front.crude import *
